{ lib, ... }:

let inherit (lib.nix2lua) nf; in
{
  plugin.editorconfig-nvim = { };

  plugins = {
    interface = {
      colorizer = {
        enable = lib.mkDefault true;

        filetypes = lib.mkDefault [ "*" ];

        settings = {
          byFiletype = lib.mkDefault {
            css = { css = true; };
            scss = { css = true; };
            sass = { css = true; };
            less = { css = true; };
          };
          overrideDefaults = lib.mkDefault {
            names = false;
          };
        };
      };
      lualine = {
        enable = lib.mkDefault true;
        settings = lib.mkDefault {
          sections = {
            lualine_a = [
              [ "filename" (nf "path" 1) ]
            ];
            lualine_b = [ "branch" "diff" "diagnostics" ];
            lualine_c = [ "lsp_progress" ];
            lualine_x = [ "filesize" "filetype" ];
            lualine_y = [ "progress" ];
            lualine_z = [ "location" "mode" ];
          };
        };
      };
    };

    language-server.lspconfig.enable = lib.mkDefault true;

    navigation = {
      hop-nvim.enable = lib.mkDefault true;
      nvim-tree.enable = lib.mkDefault true;
      telescope = {
        enable = lib.mkDefault true;
        extensions = {
          telescope-live-grep-args-nvim = { name = "live_grep_args"; };
        };
      };
    };

    snippet.luasnip.enable = lib.mkDefault true;

    style = {
      neoformat.enable = lib.mkDefault true;
      nvim-treesitter.enable = lib.mkDefault true;
    };

    theme.catppuccin = {
      enable = lib.mkDefault true;
      settings = lib.mkDefault {
        flavour = "frappe";
      };
    };

    gitsigns.enable = lib.mkDefault true;

    nvim-cmp.enable = lib.mkDefault true;
  };
}
